
package org.omegahat.DataStructures.Data;


/**
 A utility class that constructs patterns that can then be used as selectors
 for variables.  Intuitively, the selectors are patterns on the set of integers
 that can index some variable.  The methods for this class manipulate and generate such patterns.
 
*/
public class  Selector implements SelectorInt  
{
public Selector() {}
/**
 A selector containing the single index.
*/
public Selector(int i) {
  setArray(new int[] {i});
}
/**
 A selector containing the sequence between the two arguments.
*/
public Selector(int from, int to) {
  seq(from, to);
}

/**
 A selector containing the supplied array of indices.
*/
public Selector(int items[]) {
  setArray(items);
}
/**
@serial 
*/
protected int[] array;

/**
   Accessor for {@link #array array} field
 */
public int[] getArray()
{
   return(array);
}

/**
   Accessor for setting {@link #array array} field
 */
public int[] setArray(int[] value)
{
   array = value;
  return(array);
}

public int[] toArray() {
  return(getArray() == null ? setArray(new int[0]) : getArray());
}


/**
 Sets the object to be the sequence between the two arguments, ascending or descending.
*/
public Selector seq(int from, int to)  {
  int delta = 1, start = from;
  int  n = to - from +1;
  if(to < from) {n = from - to + 1; delta = -1;}
  int [] ii = setArray(new int[n]);
  for(int j=0; j<n; j++) {ii[j] = start; start += delta;}
  return(this);
}



/**
 Sets the object to be the sequence starting from the first argument, proceeding by the
 step, with the number of indices equal to length.
*/
public Selector seq(int from, int step, int length){
  int[] ii = setArray(new int[length]);
  for(int i=0; i<length; i++) {
    ii[i] = from;
    from += step;
  }
  return(this);
}

/**
 Returns a selector replicating the current object <code>times</code>. 
*/
public Selector rep(int times) {
  int[] from = toArray(); int n = from.length; 
  int[] ii = new int[n * times];
  Selector value = new Selector(ii);
  int to=0;
  for(int j=0; j<times; j++)
    for(int i=0; i<n; i++) {
      ii[to++] = from[i];
  }
  return(value);
}

/**
 Replicate each element of the selector the number of times specified by the
 corresponding element of the argument (if the argument is shorter than this object,
 only that many elements will be used).
*/
public Selector rep(int[] times) {
  int[] from = toArray();
  int n = 0, length = from.length, i;
  if(times.length < length)
    length = times.length;
  for(i=0; i<length; i++)
    n += times[i];
  int[] ii = new int[n]; int to=0;
  for(i=0; i<length; i++) {
    int val = from[i], count = times[i];
    for(int j=0; j<count; j++)
      ii[to++] = val;
  }
  return(new Selector(ii));
}
    

}

/*
Copyright (c) 1998 1999, The Omega Project for Statistical Computing.
     All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions are
     met:

       Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.

       Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

       Neither name of the Omega Project for Statistical Computing nor the 
       names of its contributors may be used to endorse or promote products 
       derived from this software without specific prior written permission.

     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY
     EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
     SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
     PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
     STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
     THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

*/

