
package org.omegahat.DataStructures.Data.Viewers;



/**
 Class used to display a {@link org.omegahat.DataStructures.Data.DataFrameInt DataFrameInt} object in a grid, potentially allowing editing, 
 selection of columns and/or rows.
*/
public class DataFrameViewer  
      extends javax.swing.JTable
      implements javax.swing.event.ChangeListener
{
/**
   
*/
protected org.omegahat.DataStructures.Data.DataFrameInt dataFrame;

public DataFrameViewer(org.omegahat.DataStructures.Data.DataFrameInt dataFrame) {
 this(dataFrame, true);
}


public DataFrameViewer(org.omegahat.DataStructures.Data.DataFrameInt dataFrame, boolean withHeaders) {
 super(new DataFrameTableModel(dataFrame));
 System.err.println("Creating DataFrameViewer. Got TableModel");
 dataFrame(dataFrame);
 initialize(withHeaders);
}

/**
  Establish renderers and editores for the different types of cells we will encounter
 ({@link java.lang.Number Number} and {@link java.lang.String String})
  and create the column headers.
*/
public void initialize() {
  initialize(true);
}

public void initialize(boolean withHeaders) {
 if(withHeaders)
   setHeaders(); 

   /*
      Since the DataFrameTableModel uses Strings for the type of a factor
      and Number for other variables, we can establish renderers for factors
      very simply by associating them with the String class, rather than having
      to dynamically lookup the variable in the DataFrame and see if it is a factor.
      We can do this if we need to use Strings for other cell type.
    */
 FactorCellRenderer cellRenderer = new FactorCellRenderer(dataFrame());
 setDefaultRenderer(String.class, cellRenderer);

 FactorCellEditor editor = new FactorCellEditor();
  editor.addCellEditorListener(this);
 setDefaultEditor(String.class, editor);

  getColumnModel().setColumnSelectionAllowed(true);
}

/**
 Create the column headers for the display.
*/
public int setHeaders() {
 javax.swing.table.TableColumnModel model = getColumnModel();
 int n = model.getColumnCount();
  javax.swing.table.TableCellRenderer headerRenderer = getHeaderRenderer();
  for(int i = 0 ;i < n; i++) {
     model.getColumn(i).setHeaderRenderer(headerRenderer);
  }

 return(n);
}


public javax.swing.table.TableCellRenderer getHeaderRenderer() {
  return(new VariableNameRenderer(getColumnModel().getSelectionModel()));
}



public void stateChanged(javax.swing.event.ChangeEvent ev) {
 FactorCellEditor  box = (FactorCellEditor) ev.getSource();
System.err.println("Edited cell " + box.activeObservation());
  box.activeVariable().add(box.getSelectedItem(), box.activeObservation());
}


public org.omegahat.DataStructures.Data.DataFrameInt dataFrame()
{
   return(dataFrame);
}

public org.omegahat.DataStructures.Data.DataFrameInt dataFrame(org.omegahat.DataStructures.Data.DataFrameInt value)
{
   dataFrame = value;
  return(dataFrame);
}



}

/*
Copyright (c) 1998 1999, The Omega Project for Statistical Computing.
     All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions are
     met:

       Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.

       Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

       Neither name of the Omega Project for Statistical Computing nor the 
       names of its contributors may be used to endorse or promote products 
       derived from this software without specific prior written permission.

     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY
     EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
     SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
     PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
     STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
     THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

*/

